import opentracing
from bottle import default_app
from signalfx_tracing import utils
from wrapt import wrap_function_wrapper

from bottle_opentracing import BottleTracing

config = utils.Config(
    trace_all=True,
    traced_attributes=['path', 'method'],
    tracer=None,
)


def instrument(tracer=None):
    bottle = utils.get_module('bottle')
    if utils.is_instrumented(bottle):
        return

    def bottle_tracer(run, _, args, kwargs):
        """
        A function wrapper of bottle.run to create a corresponding
        BottleTracer upon app instantiation.
        """
        app = kwargs.get('app', default_app())
        _tracer = tracer or config.tracer or opentracing.tracer
        BottleTracing(tracer=_tracer, trace_all_requests=config.trace_all,
                      app=app, traced_attributes=config.traced_attributes)

        run(**kwargs)

    wrap_function_wrapper('bottle', 'run', bottle_tracer)
    utils.mark_instrumented(bottle)


def uninstrument():
    """
    Will only prevent new applications from registering tracers.
    It's not reasonably feasible to remove existing before/after_request
    trace methods of existing apps.
    """
    bottle = utils.get_module('bottle')
    if not utils.is_instrumented(bottle):
        return

    utils.revert_wrapper(bottle, 'run')
    utils.mark_uninstrumented(bottle)



import bottle
import opentracing
from bottle import Bottle
from opentracing.ext import tags

from bottle_opentracing import BottleTracing


class APMBottleApp(Bottle):
    def default_error_handler(self, res):
        self.handle_error(res)
        return super().default_error_handler(res)

    @classmethod
    def handle_error(cls, res):
        _tracer = opentracing.tracer
        if _tracer:
            request = bottle.request
            with _tracer.start_active_span(request.path):
                BottleTracing.add_request_tags(_tracer.active_span, request)
                BottleTracing.add_response_tags(_tracer.active_span, res)
                cls.notice_error(_tracer, res.exception)

    @classmethod
    def notice_error(cls, tracer, err: Exception):
        if tracer and tracer.active_span:
            tracer.active_span.set_tag(tags.ERROR, True)
            tracer.active_span.log_kv({
                'event': tags.ERROR,
                'error.object': err,
            })



import unittest
from unittest.mock import patch

import bottle
import opentracing

from bottle_opentracing import BottleTracing


class TestValues(unittest.TestCase):
    def test_tracer(self):
        tracer = opentracing.Tracer()
        tracing = BottleTracing(tracer)
        self.assertIs(tracing.tracer, tracer)
        self.assertFalse(tracing._trace_all_requests)

    def test_global_tracer(self):
        tracing = BottleTracing()
        with patch('opentracing.tracer'):
            self.assertIs(tracing.tracer, opentracing.tracer)
            opentracing.tracer = object()
            self.assertIs(tracing.tracer, opentracing.tracer)

    def test_trace_all_requests(self):
        app = bottle.app()
        tracing = BottleTracing(app=app)
        self.assertTrue(tracing._trace_all_requests)

        tracing = BottleTracing(app=app, trace_all_requests=False)
        self.assertFalse(tracing._trace_all_requests)

    def test_trace_all_requests_no_app(self):
        # when trace_all_requests is True, an app object is *required*
        self.assertRaises(ValueError, lambda: BottleTracing(trace_all_requests=True))

    def test_start_span_invalid(self):
        self.assertRaises(ValueError, lambda: BottleTracing(start_span_cb=0))



from setuptools import setup

'''
Bottle-OpenTracing
-----------------

This extension provides simple integration of OpenTracing in Bottle applications.
'''
setup(
    name='Bottle-OpenTracing',
    version='0.0.5',
    url='https://github.com/Creativelair/Bottle-Opentracing',
    download_url='https://github.com/Creativelair/Bottle-Opentracing/archive/v0.0.5.tar.gz',
    license='BSD',
    author='Daniel Jimenez',
    author_email='danijimenez010@gmail.com',
    description='OpenTracing support for Bottle applications',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    packages=['bottle_opentracing', 'tests'],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=[
        'bottle',
        'opentracing>=2.0,<3',
    ],
    extras_require={
        'tests': [
            'flake8',
            'flake8-quotes',
            'mock',
            'pytest',
            'pytest-cov',
        ],
    },
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



import unittest
from unittest.mock import patch

import bottle
import opentracing
from opentracing.ext import tags
from opentracing.mocktracer import MockTracer
from webtest import TestApp

from bottle_opentracing import BottleTracing

app = bottle.app()
test_app = TestApp(app)
after_hooks = app._hooks['after_request']

tracing_all = BottleTracing(MockTracer(), True, app, ['url'])
tracing = BottleTracing(MockTracer())
tracing_deferred = BottleTracing(lambda: MockTracer(), True, app, ['url'])


def flush_spans(tcr):
    for req in tcr._current_scopes:
        tcr._current_scopes[req].close()
    tcr._current_scopes = {}


@bottle.get('/test')
def check_test_works():
    return 'Success'


@bottle.get('/another_test')
@tracing.trace('url', 'url_rule')
def decorated_fn():
    return 'Success again'


@bottle.get('/another_test_simple')
@tracing.trace('query_string', 'is_xhr')
def decorated_fn_simple():
    return 'Success again'


@bottle.get('/error_test')
@tracing.trace()
def decorated_fn_with_error():
    raise RuntimeError('Should not happen')


@bottle.get('/decorated_child_span_test')
@tracing.trace()
def decorated_fn_with_child_span():
    with tracing.tracer.start_active_span('child'):
        return 'Success'


@bottle.get('/wire')
def send_request():
    span = tracing_all.get_span()
    headers = {}
    tracing_all.tracer.inject(span.context,
                              opentracing.Format.TEXT_MAP,
                              headers)
    test_app.get('/test', headers=headers)
    return ''


class TestTracing(unittest.TestCase):
    def setUp(self):
        tracing_all.tracer.reset()
        tracing.tracer.reset()
        tracing_deferred.tracer.reset()

    def test_span_creation(self):
        app._hooks['after_request'] = []

        test_app.get('/test')

        self.assertTrue(tracing_all.get_span(bottle.request))
        self.assertFalse(tracing.get_span(bottle.request))
        self.assertTrue(tracing_deferred.get_span(bottle.request))

        active_span = tracing_all.tracer.active_span
        self.assertIs(tracing_all.get_span(bottle.request), active_span)

        flush_spans(tracing_all)
        flush_spans(tracing_deferred)

    def test_span_deletion(self):
        app._hooks['after_request'] = after_hooks

        self.assertFalse(tracing_all._current_scopes)
        self.assertFalse(tracing_deferred._current_scopes)
        test_app.get('/test')
        self.assertFalse(tracing_all._current_scopes)
        self.assertFalse(tracing_deferred._current_scopes)

    def test_span_tags(self):
        test_app.get('/another_test_simple')

        spans = tracing.tracer.finished_spans()
        self.assertEqual(1, len(spans))
        self.assertEqual(spans[0].tags, {
            tags.COMPONENT: 'Bottle',
            tags.HTTP_METHOD: 'GET',
            tags.SPAN_KIND: tags.SPAN_KIND_RPC_SERVER,
            tags.HTTP_URL: 'http://localhost:80/another_test_simple',
            tags.HTTP_STATUS_CODE: 200,
            'is_xhr': 'False',
        })

    def test_requests_distinct(self):
        app._hooks['after_request'] = []

        test_app.get('/test')

        second_scope = tracing_all._current_scopes.pop(bottle.request)
        self.assertTrue(second_scope)
        second_scope.close()
        self.assertFalse(tracing_all.get_span(bottle.request))

        flush_spans(tracing_all)
        flush_spans(tracing_deferred)

    def test_decorator(self):
        app._hooks['after_request'] = []

        test_app.get('/another_test')

        self.assertFalse(tracing.get_span(bottle.request))
        self.assertEqual(1, len(tracing_deferred._current_scopes))
        self.assertEqual(1, len(tracing_all._current_scopes))

        active_span = tracing_all.tracer.active_span
        self.assertIs(tracing_all.get_span(bottle.request), active_span)

        flush_spans(tracing)
        flush_spans(tracing_all)
        flush_spans(tracing_deferred)

        app._hooks['after_request'] = after_hooks

        test_app.get('/another_test')

        self.assertFalse(tracing_all._current_scopes)
        self.assertFalse(tracing._current_scopes)
        self.assertFalse(tracing_deferred._current_scopes)

    def test_decorator_trace_all(self):
        # Fake we are tracing all, which should disable
        # tracing through our decorator.
        with patch.object(tracing, '_trace_all_requests', new=True):
            rv = test_app.get('/another_test_simple')
            self.assertTrue('200' in str(rv.status_code))

        spans = tracing.tracer.finished_spans()
        self.assertEqual(0, len(spans))

    def test_over_wire(self):
        rv = test_app.get('/wire')
        self.assertTrue('200' in str(rv.status_code))

        spans = tracing_all.tracer.finished_spans() + [tracing_all.tracer.active_span]
        self.assertEqual(2, len(spans))
        self.assertEqual(spans[0].context.trace_id, spans[1].context.trace_id)
        self.assertEqual(spans[0].parent_id, spans[1].context.span_id)

    def test_child_span(self):
        rv = test_app.get('/decorated_child_span_test')
        self.assertTrue('200' in str(rv.status_code))

        spans = tracing.tracer.finished_spans()
        self.assertEqual(2, len(spans))
        self.assertEqual(spans[0].context.trace_id, spans[1].context.trace_id)
        self.assertEqual(spans[0].parent_id, spans[1].context.span_id)


class TestTracingStartSpanCallback(unittest.TestCase):
    def test_simple(self):
        def start_span_cb(span, request):
            span.set_tag('component', 'not-bottle')
            span.set_tag('mytag', 'myvalue')

        tracing = BottleTracing(MockTracer(), True, app, start_span_cb=start_span_cb)
        rv = test_app.get('/test')
        self.assertTrue('200' in str(rv.status_code))

        spans = tracing.tracer.finished_spans()
        self.assertEqual(1, len(spans))
        self.assertEqual(spans[0].tags.get(tags.COMPONENT, None), 'not-bottle')
        self.assertEqual(spans[0].tags.get('mytag', None), 'myvalue')

    def test_error(self):
        def start_span_cb(span, request):
            raise RuntimeError('Should not happen')

        tracing = BottleTracing(MockTracer(), True, app, start_span_cb=start_span_cb)
        rv = test_app.get('/test')
        self.assertTrue('200' in str(rv.status_code))

        spans = tracing.tracer.finished_spans()
        self.assertEqual(1, len(spans))
        self.assertIsNone(spans[0].tags.get(tags.ERROR, None))
